#! /usr/bin/python
import os
import sys
import unittest
import json
import subprocess


class Util_test(unittest.TestCase):
    maxDiff = None

    def test_example(self):
        self.assertEqual(1,1) 

    def test_os_popen_example(self):
        cmd = 'echo example'
        with os.popen(cmd) as process:
            result = process.read()
        self.assertEqual(result, 'example\n')

    def test_help(self):        
        cmd = 'python src/match.py -h'
        with os.popen(cmd) as process:
            result = process.read()
        expected_output = (
""" _______________________________________________________________________________________________________________________ 
|***********************************************************************************************************************|
| * List all available commands.                                                                                        |
| ./src/match.py -h                                                                                                     |
|_______________________________________________________________________________________________________________________|
| * Make prediction with all input parameters in JSON file                                                              |
| ./src/match.py  -j [input_json_file]                                                                                  |
| Example: python ./src/match.py  -j ./examples/pepmatch.json                                                           |
|_______________________________________________________________________________________________________________________|
| * Make prediction with all input parameters in JSON file and write result to a JSON file                              |
| ./src/match.py  -j [input_json_file]  -o [output_prefix] -f json                                                      |
| Example: python ./src/match.py  -j examples/pepmatch.json -o output -f json                                           |
|_______________________________________________________________________________________________________________________|
| * split the input JSON file into atomic units, and create a job description file as output which includes commands    |
|   for all aotomic unit preidction and the aggreestion                                                                 |
| src/match.py -j [input_json_file] --split --split-dir=[atomic_units_json_file_dir]                                    |
| Example: src/match.py -j examples/pepmatch.json --split --split-dir=examples/job/parameter_units                      |
|_______________________________________________________________________________________________________________________|
| * result file aggreestion after predictions are done                                                                  |
| src/match.py --aggregate --job-desc-file=[job description file]                                                      \\
        --aggregate-input-dir=[atomic_units_json_file_dir]   --aggregate-result-dir=[aggregate output path]             |
| Example: python src/match.py --aggregate --job-desc-file=examples/job/job_descriptions.json                          \\
        --aggregate-input-dir=examples/job/results --aggregate-result-dir=examples/job/aggregate                        |
|_______________________________________________________________________________________________________________________|
"""
        )
        self.assertEqual(result, expected_output)
        # if not options are given, help info will be returned as well
        cmd = 'python src/match.py'
        with os.popen(cmd) as process:
            result = process.read()
        self.assertEqual(result, expected_output)



class TestCommonPrediction(unittest.TestCase):
    maxDiff = None

    def test_mismatch(self):

        cmd="python src/match.py  -j test_data/mismatch/input.json -o test_data/mismatch/output -f json"
        with os.popen(cmd) as process:
            printed_result = process.read()
        with open('test_data/mismatch/output.json','r') as f:
            result = json.load(f)
        with open('test_data/mismatch/expected_output.json','r') as f:
            expected_output = json.load(f)
        if "results" in result:
            result = result["results"][0]
        self.assertEqual(sorted(result['table_data']), sorted(expected_output['table_data']))

    def test_best_match(self):

        cmd="python src/match.py  -j test_data/best_match/input.json -o test_data/best_match/output -f json"
        with os.popen(cmd) as process:
            printed_result = process.read()
        with open('test_data/best_match/output.json','r') as f:
            result = json.load(f)
        with open('test_data/best_match/expected_output.json','r') as f:
            expected_output = json.load(f)
        if "results" in result:
            result = result["results"][0]
        self.assertEqual(sorted(result['table_data']), sorted(expected_output['table_data']))

    def test_exact_match(self):

        cmd="python src/match.py  -j test_data/exact_match/input.json -o test_data/exact_match/output -f json"
        with os.popen(cmd) as process:
            printed_result = process.read()
        with open('test_data/exact_match/output.json','r') as f:
            result = json.load(f)
        with open('test_data/exact_match/expected_output.json','r') as f:
            expected_output = json.load(f)
        if "results" in result:
            result = result["results"][0]
        self.assertEqual(sorted(result['table_data']), sorted(expected_output['table_data']))

    def test_no_match(self):

        cmd="python src/match.py  -j test_data/no_match/input.json -o test_data/no_match/output -f json"
        with os.popen(cmd) as process:
            printed_result = process.read()
        with open('test_data/no_match/output.json','r') as f:
            result = json.load(f)
        with open('test_data/no_match/expected_output.json','r') as f:
            expected_output = json.load(f)
        if "results" in result:
            result = result["results"][0]
        self.assertEqual(sorted(result['table_data']), sorted(expected_output['table_data']))

def test():    
    unittest.main()

def main():
    print( 'run PepMatch tests...')
    test()

if __name__ == '__main__':   
    main()

